@@ -1,13 +1,25 @@
-import ast
+import ast, inspect, os
+import textwrap
 
 from numba import *
 from . import naming
 from .minivect import minitypes
 import numba.ast_translate as translate
-import numba.ast_type_inference as type_inference
 from numba import nodes
+from llpython.byte_translator import LLVMTranslator
+
+import logging
+import traceback
+logger = logging.getLogger(__name__)
+
+try:
+    from meta.decompiler import decompile_func
+except Exception, exn:
+    logger.warn("Could not import Meta - AST translation will not work "
+                "if the source is not available:\n%s" %
+                traceback.format_exc())
+    decompile_func = None
 
-import meta.decompiler
 import llvm.core
 
 def is_numba_func(func):
@@ -28,17 +40,66 @@ def fix_ast_lineno(tree):
 
     return tree
 
+## Fixme: 
+##  This should be changed to visit the AST and fix-up where a None object
+##  is present as this will likely not work for all AST.
+def _fix_ast(myast):
+    import _ast
+
+    # Remove Pass nodes from the end of the ast
+    while len(myast.body) > 0  and isinstance(myast.body[-1], _ast.Pass):
+        del myast.body[-1]
+    # Add a return node at the end of the ast if not present
+    if len(myast.body) < 1 or not isinstance(myast.body[-1], _ast.Return):
+        name = _ast.Name(id='None',ctx=_ast.Load(), lineno=0, col_offset=0)
+        myast.body.append(ast.Return(name))
+    # remove _decorator list which sometimes confuses ast visitor
+    try:
+        indx = myast._fields.index('decorator_list')
+    except ValueError:
+        return
+    else:
+        myast.decorator_list = []
+
 def _get_ast(func):
-    return meta.decompiler.decompile_func(func)
+    if os.environ.get('NUMBA_FORCE_META_AST'):
+        func_def = decompile_func(func)
+        assert isinstance(func_def, ast.FunctionDef)
+        return func_def
+    try:
+        source = inspect.getsource(func)
+    except IOError:
+        return decompile_func(func)
+    else:
+        source = textwrap.dedent(source)
+        if source.startswith('@'):
+            decorator, sep, source = source.partition('\n')
+            while not source.startswith('def'): # decorator can have multiple lines
+                decorator, sep, source = source.partition('\n')
+        module_ast = ast.parse(source)
+
+        # fix line numbering
+        lineoffset = func.func_code.co_firstlineno
+        ast.increment_lineno(module_ast, lineoffset)
+
+        assert len(module_ast.body) == 1
+        func_def = module_ast.body[0]
+        _fix_ast(func_def)
+        assert isinstance(func_def, ast.FunctionDef)
+        return func_def
+
+def _infer_types(context, func, restype=None, argtypes=None, **kwargs):
+    import numba.ast_type_inference as type_inference
 
-def _infer_types(context, func, restype=None, argtypes=None):
     ast = _get_ast(func)
     func_signature = minitypes.FunctionType(return_type=restype,
                                             args=argtypes)
-    return type_inference.run_pipeline(context, func, ast, func_signature)
+    return type_inference.run_pipeline(context, func, ast,
+                                       func_signature, **kwargs)
 
 
-def _compile(context, func, restype=None, argtypes=None, **kwds):
+def _compile(context, func, restype=None, argtypes=None, ctypes=False,
+             compile_only=False, name=None, **kwds):
     """
     Compile a numba annotated function.
 
@@ -46,23 +107,31 @@ def _compile(context, func, restype=None, argtypes=None, **kwds):
         - run type inference using the given input types
         - compile the function to LLVM
     """
-    func_signature, symtab, ast = _infer_types(context, func, restype, argtypes)
-    func_name = naming.specialized_mangle(func.__name__, func_signature.args)
+    func_signature, symtab, ast = _infer_types(context, func,
+                                               restype, argtypes, **kwds)
+    func_name = name or naming.specialized_mangle(func.__name__, func_signature.args)
+    func_signature.name = func_name
 
     t = translate.LLVMCodeGenerator(
         context, func, ast, func_signature=func_signature,
-        func_name=func_name, symtab=symtab, **kwds)
+        symtab=symtab, **kwds)
     t.translate()
 
-    return func_signature, t.lfunc, t.get_ctypes_func(kwds.get('llvm', True))
+    if compile_only:
+        return func_signature, t.lfunc, None
+    if ctypes:
+        ctypes_func = t.get_ctypes_func(kwds.get('llvm', True))
+        return func_signature, t.lfunc, ctypes_func
+    else:
+        return func_signature, t.lfunc, t.build_wrapper_function()
 
 class FunctionCache(object):
     """
     Cache for compiler functions, declared external functions and constants.
     """
-    def __init__(self, context):
+    def __init__(self, context, module=None):
         self.context = context
-        self.module = context.llvm_context.get_default_module()
+        self.module = module or context.llvm_context.get_default_module()
 
         # All numba-compiled functions
         # (py_func, arg_types) -> (signature, llvm_func, ctypes_func)
@@ -86,7 +155,8 @@ class FunctionCache(object):
 
         return result
 
-    def compile_function(self, func, argtypes, restype=None, **kwds):
+    def compile_function(self, func, argtypes, restype=None,
+                         ctypes=False, **kwds):
         """
         Compile a python function given the argument types. Compile only
         if not compiled already, and only if an annotated numba function
@@ -100,61 +170,79 @@ class FunctionCache(object):
         if func is not None:
             result = self.get_function(func, argtypes)
             if result is not None:
-                return result
+                sig, trans, pycall = result
+                return sig, trans.lfunc, pycall
 
             if is_numba_func(func):
+                from numba import pipeline
+
                 func = getattr(func, '_numba_func', func)
+                compile_only = getattr(func, '_numba_compile_only', False)
+                kwds['compile_only'] = kwds.get('compile_only', compile_only)
                 # numba function, compile
-                func_signature, lfunc, ctypes_func = _compile(
-                                self.context, func, restype, argtypes, **kwds)
+                func_signature, translator, ctypes_func = pipeline.compile(
+                                self.context, func, restype, argtypes,
+                                ctypes=ctypes, **kwds)
                 self.compiled_functions[func, tuple(func_signature.args)] = (
-                                            func_signature, lfunc, ctypes_func)
-                return func_signature, lfunc, ctypes_func
+                                      func_signature, translator, ctypes_func)
+                return func_signature, translator.lfunc, ctypes_func
 
         # print func, getattr(func, '_is_numba_func', False)
         # create a signature taking N objects and returning an object
         signature = ofunc(argtypes=ofunc.arg_types * len(argtypes)).signature
         return signature, None, func
 
-    def function_by_name(self, name):
+    def function_by_name(self, name, module=None, **kws):
         """
         Return the signature and LLVM function given a name. The function must
         either already be compiled, or it must be defined in this module.
         """
         if name in self.external_functions:
-            return self.external_functions[name]
-        else:
-            declared_func = globals()[name]()
-            lfunc = self.build_function(declared_func)
-            return declared_func.signature, lfunc
+            extfunc = self.external_functions[name]
+            if module is None or extfunc.module == module:
+                return extfunc
+
+        declared_func = globals()[name](**kws)
+        lfunc = self.build_function(declared_func, module=module)
+        return declared_func.signature, lfunc
 
     def call(self, name, *args, **kw):
-        temp_name = kw.get('temp_name', '')
-        function_cls = globals()[name]
-        sig, lfunc = self.function_by_name(name)
-        return nodes.NativeCallNode(sig, args, lfunc, name=temp_name)
+        temp_name = kw.pop('temp_name', name)
+        sig, lfunc = self.function_by_name(name, **kw)
 
-    def build_function(self, external_function):
+        if name in globals():
+            external_func = globals()[name]
+            exc_check = dict(badval=external_func.badval,
+                             goodval=external_func.goodval,
+                             exc_msg=external_func.exc_msg,
+                             exc_type=external_func.exc_type,
+                             exc_args=external_func.exc_args)
+        else:
+            exc_check = {}
+
+        result = nodes.NativeCallNode(sig, args, lfunc, name=temp_name,
+                                      **exc_check)
+        return result
+
+    def build_function(self, external_function, module=None):
         """
         Build a function given it's signature information. See the
         `ExternalFunction` class.
         """
+        module = module or self.module
         try:
-            lfunc = self.module.get_function_named(external_function.name)
+            lfunc = module.get_function_named(external_function.name)
         except llvm.LLVMException:
             func_type = minitypes.FunctionType(
                     return_type=external_function.return_type,
                     args=external_function.arg_types,
                     is_vararg=external_function.is_vararg)
             lfunc_type = func_type.to_llvm(self.context)
-            lfunc = self.module.add_function(lfunc_type, external_function.name)
+            lfunc = module.add_function(lfunc_type, external_function.name)
 
             if external_function.linkage == llvm.core.LINKAGE_INTERNAL:
-                entry = lfunc.append_basic_block('entry')
-                builder = lc.Builder.new(entry)
                 lfunc.linkage = external_function.linkage
-                external_function.implementation(
-                                        self.module, builder, lfunc)
+                external_function.implementation(module, lfunc)
 
         return lfunc
 
@@ -162,11 +250,11 @@ class FunctionCache(object):
         if (module, const_str) in self.string_constants:
             ret_val = self.string_constants[(module, const_str)]
         else:
-            lconst_str = lc.Constant.stringz(const_str)
+            lconst_str = llvm.core.Constant.stringz(const_str)
             ret_val = module.add_global_variable(lconst_str.type, "__STR_%d" %
                                                  (len(self.string_constants),))
             ret_val.initializer = lconst_str
-            ret_val.linkage = lc.LINKAGE_INTERNAL
+            ret_val.linkage = llvm.core.LINKAGE_INTERNAL
             self.string_constants[(module, const_str)] = ret_val
 
         return ret_val
@@ -244,7 +332,7 @@ class _LLVMModuleUtils(object):
             res = translator.builder.load(
                 translator.builder.load(
                     translator.builder.gep(args[0]._llvm, [
-                        _int32_zero, lc.Constant.int(_int32, shape_ofs)])))
+                        _int32_zero, llvm.core.Constant.int(_int32, shape_ofs)])))
         else:
             return cls.build_object_len(translator, args)
 
@@ -261,7 +349,7 @@ class _LLVMModuleUtils(object):
         largs = [translator.builder.load(
             translator.builder.gep(larr, [
                 _int32_zero,
-                lc.Constant.int(_int32,
+                llvm.core.Constant.int(_int32,
                                 _numpy_array_field_ofs[field_name])]))
                  for field_name in ('ndim', 'shape', 'descr')]
         largs.append(_int32_zero)
@@ -285,7 +373,7 @@ class _LLVMModuleUtils(object):
         larg = args[0]._llvm
         elem_ltyp = _float if larg.type == _complex64 else _double
         new_imag_lval = translator.builder.fmul(
-            lc.Constant.real(elem_ltyp, -1.),
+            llvm.core.Constant.real(elem_ltyp, -1.),
             translator.builder.extract_value(larg, 1))
         assert hasattr(translator.builder, 'insert_value'), (
             "llvm-py support for LLVMBuildInsertValue() required to build "
@@ -304,6 +392,12 @@ class ExternalFunction(object):
     linkage = None
     is_vararg = False
 
+    badval = None
+    goodval = None
+    exc_type = None
+    exc_msg = None
+    exc_args = None
+
     def __init__(self, **kwargs):
         vars(self).update(kwargs)
 
@@ -320,7 +414,7 @@ class ExternalFunction(object):
         else:
             return self.func_name
 
-    def implementation(self, module, builder, lfunc):
+    def implementation(self, module, lfunc):
         return None
 
 class InternalFunction(ExternalFunction):
@@ -405,29 +499,184 @@ class PySlice_New(ExternalFunction):
     arg_types = [object_, object_, object_]
     return_type = object_
 
+class PyErr_SetString(ExternalFunction):
+    arg_types = [object_, c_string_type]
+    return_type = void
+
+class PyErr_Format(ExternalFunction):
+    arg_types = [object_, c_string_type]
+    return_type = void.pointer() # object_
+    is_vararg = True
+
+class PyErr_Occurred(ExternalFunction):
+    arg_types = []
+    return_type = void.pointer() # object_
+
+class PyErr_Clear(ExternalFunction):
+    arg_types = []
+    return_type = void
+
 class PyModulo(InternalFunction):
 
     @property
     def name(self):
-        return naming.specialized_mangle('__py_modulo_%s', self.arg_types)
+        return naming.specialized_mangle('__py_modulo', self.arg_types)
 
-    def implementation(self, module, builder, ret_val):
-        different_sign_block = ret_val.append_basic_block('different_sign')
-        join_block = ret_val.append_basic_block('join')
-        arg2 = ret_val.args[1]
-        srem = builder.srem(ret_val.args[0], arg2)
-        width = int(typ[1:])
-        lbits_shifted = lc.Constant.int(str_to_llvmtype(typ), width - 1)
-        srem_sign = builder.lshr(srem, lbits_shifted)
-        arg2_sign = builder.lshr(arg2, lbits_shifted)
-        same_sign = builder.icmp(lc.ICMP_EQ, srem_sign, arg2_sign)
-        builder.cbranch(same_sign, join_block, different_sign_block)
-        builder.position_at_end(different_sign_block)
-        different_sign_res = builder.add(srem, arg2)
-        builder.branch(join_block)
-        builder.position_at_end(join_block)
-        res = builder.phi(ltyp)
-        res.add_incoming(srem, entry_block)
-        res.add_incoming(different_sign_res, different_sign_block)
-        builder.ret(res)
+    def implementation(self, module, ret_val):
+        _rtype = ret_val.type.pointee.return_type
+        _rem = (llvm.core.Builder.srem
+               if _rtype.kind == llvm.core.TYPE_INTEGER
+               else llvm.core.Builder.frem)
+        def _py_modulo (arg1, arg2):
+            ret_val = rem(arg1, arg2)
+            if ret_val < rtype(0):
+                if arg2 > rtype(0):
+                    ret_val += arg2
+            elif arg2 < rtype(0):
+                ret_val += arg2
+            return ret_val
+        LLVMTranslator(module).translate(
+            _py_modulo, llvm_function = ret_val,
+            env = {'rtype' : _rtype, 'rem' : _rem})
         return ret_val
+
+class CStringSlice2 (InternalFunction):
+    arg_types = [c_string_type, c_string_type, size_t, Py_ssize_t, Py_ssize_t]
+    return_type = void
+
+    def implementation(self, module, ret_val):
+        # logger.debug((module, str(ret_val)))
+        def _py_c_string_slice(out_string, in_string, in_str_len, lower,
+                               upper):
+            zero = lc_size_t(0)
+            if lower < zero:
+                lower += in_str_len
+            if upper < zero:
+                upper += in_str_len
+            elif upper > in_str_len:
+                upper = in_str_len
+            temp_len = upper - lower
+            if temp_len < zero:
+                temp_len = zero
+            strncpy(out_string, in_string + lower, temp_len)
+            out_string[temp_len] = li8(0)
+            return
+        LLVMTranslator(module).translate(_py_c_string_slice,
+                                         llvm_function = ret_val)
+        return ret_val
+
+class CStringSlice2Len(InternalFunction):
+    arg_types = [c_string_type, size_t, Py_ssize_t, Py_ssize_t]
+    return_type = size_t
+
+    def implementation(self, module, ret_val):
+        def _py_c_string_slice_len(in_string, in_str_len, lower, upper):
+            zero = lc_size_t(0)
+            if lower < zero:
+                lower += in_str_len
+            if upper < zero:
+                upper += in_str_len
+            elif upper > in_str_len:
+                upper = in_str_len
+            temp_len = upper - lower
+            if temp_len < zero:
+                temp_len = zero
+            return temp_len + lc_size_t(1)
+        LLVMTranslator(module).translate(_py_c_string_slice_len,
+                                         llvm_function = ret_val)
+        return ret_val
+
+class labs(ExternalFunction):
+    arg_types = [long_]
+    return_type = long_
+
+class llabs(ExternalFunction):
+    arg_types = [longlong]
+    return_type = longlong
+
+class atoi(ExternalFunction):
+    arg_types = [c_string_type]
+    return_type = int_
+
+class atol(ExternalFunction):
+    arg_types = [c_string_type]
+    return_type = long_
+
+class atoll(ExternalFunction):
+    arg_types = [c_string_type]
+    return_type = longlong
+
+class atof(ExternalFunction):
+    arg_types = [c_string_type]
+    return_type = double
+
+class strlen(ExternalFunction):
+    arg_types = [c_string_type]
+    return_type = size_t
+
+#
+### Object conversions to native types
+#
+def create_func(name, restype, argtype, d):
+    class PyLong_FromLong(ExternalFunction):
+        arg_types = [argtype]
+        return_type = restype
+
+    PyLong_FromLong.__name__ = name
+    if restype.is_object:
+        type = argtype
+    else:
+        type = restype
+
+    d[type] = PyLong_FromLong
+    globals()[name] = PyLong_FromLong
+
+_as_long = {}
+def as_long(name, type):
+    create_func(name, type, object_, _as_long)
+
+as_long('PyLong_AsLong', long_)
+as_long('PyLong_AsUnsignedLong', ulong)
+as_long('PyLong_AsLongLong', longlong)
+as_long('PyLong_AsUnsignedLongLong', ulonglong)
+#as_long('PyLong_AsSize_t', size_t) # new in py3k
+as_long('PyLong_AsSsize_t', Py_ssize_t)
+
+class PyFloat_FromDouble(ExternalFunction):
+    arg_types = [double]
+    return_type = object_
+
+class PyComplex_FromCComplex(ExternalFunction):
+    arg_types = [complex128]
+    return_type = object_
+
+class PyInt_FromString(ExternalFunction):
+    arg_types = [c_string_type, c_string_type.pointer(), int_]
+    return_type = object_
+
+class PyFloat_FromString(ExternalFunction):
+    arg_types = [object_, c_string_type.pointer()]
+    return_type = object_
+
+#
+### Conversion of native types to object
+#
+_from_long = {}
+def from_long(name, type):
+    create_func(name, object_, type, _from_long)
+
+
+from_long('PyLong_FromLong', long_)
+from_long('PyLong_FromUnsignedLong', ulong)
+from_long('PyLong_FromLongLong', longlong)
+from_long('PyLong_FromUnsignedLongLong', ulonglong)
+from_long('PyLong_FromSize_t', size_t) # new in 2.6
+from_long('PyLong_FromSsize_t', Py_ssize_t)
+
+class PyFloat_AsDouble(ExternalFunction):
+    arg_types = [object_]
+    return_type = double
+
+class PyComplex_AsCComplex(ExternalFunction):
+    arg_types = [object_]
+    return_type = complex128
