@@ -32,29 +32,26 @@ __all__ = ['Py_ssize_t', 'void', 'char', 'uchar', 'short', 'ushort',
            'size_t', 'npy_intp', 'c_string_type', 'bool_', 'object_',
            'float_', 'double', 'longdouble', 'float32', 'float64', 'float128',
            'int8', 'int16', 'int32', 'int64', 'uint8', 'uint16', 'uint32', 'uint64',
-           'complex64', 'complex128', 'complex256']
+           'complex64', 'complex128', 'complex256', 'struct', 'Py_uintptr_t']
 
 import sys
 import math
 import copy
-import ctypes
+import struct as struct_
+import types
 import textwrap
 
-try:
-    import llvm.core
-    from llvm import core as lc
-except ImportError:
-    llvm = None
-
 import miniutils
 import minierror
 
-# Check below taken from Numba
-if sys.maxint > 2**33:
-    _plat_bits = 64
-else:
-    _plat_bits = 32
+from miniutils import *
 
+_plat_bits = struct_.calcsize('@P') * 8
+
+if struct_.pack('i', 1)[0] == '\1':
+    nbo = '<' # little endian
+else:
+    nbo = '>' # big endian
 
 class TypeMapper(object):
     """
@@ -159,13 +156,13 @@ class TypeMapper(object):
 
     def promote_numeric(self, type1, type2):
         "Promote two numeric types"
-        return max([type1, type2], key=lambda type: type.rank)
+        return miniutils.max([type1, type2], key=lambda type: type.rank)
 
     def promote_arrays(self, type1, type2):
         "Promote two array types in an expression to a new array type"
         equal_ndim = type1.ndim == type2.ndim
         return ArrayType(self.promote_types(type1.dtype, type2.dtype),
-                         ndim=max(type1.ndim, type2.ndim),
+                         ndim=miniutils.max((type1.ndim, type2.ndim)),
                          is_c_contig=(equal_ndim and type1.is_c_contig and
                                       type2.is_c_contig),
                          is_f_contig=(equal_ndim and type1.is_f_contig and
@@ -173,6 +170,7 @@ class TypeMapper(object):
 
     def promote_types(self, type1, type2):
         "Promote two arbitrary types"
+        string_types = c_string_type, char.pointer()
         if type1.is_pointer and type2.is_int_like:
             return type1
         elif type2.is_pointer and type2.is_int_like:
@@ -183,6 +181,8 @@ class TypeMapper(object):
             return self.promote_numeric(type1, type2)
         elif type1.is_array and type2.is_array:
             return self.promote_arrays(type1, type2)
+        elif type1 in string_types and type2 in string_types:
+            return c_string_type
         else:
             raise minierror.UnpromotableTypeError((type1, type2))
 
@@ -202,6 +202,12 @@ def map_dtype(dtype):
     >>> map_dtype(np.dtype(np.complex128))
     complex128
     """
+    import numpy as np
+
+    if dtype.byteorder not in ('=', nbo, '|') and dtype.kind in ('iufbc'):
+        raise minierror.UnmappableTypeError(
+                "Only native byteorder is supported", dtype)
+
     item_idx = int(math.log(dtype.itemsize, 2))
     if dtype.kind == 'i':
         return [int8, int16, int32, int64][item_idx]
@@ -225,6 +231,12 @@ def map_dtype(dtype):
             return complex128
         elif dtype.itemsize == 32:
             return complex256
+    elif dtype.kind == 'V':
+        fields = [(name, map_dtype(dtype.fields[name][0]))
+                      for name in dtype.names]
+        is_aligned = dtype.alignment != 1
+        return struct(fields, packed=not getattr(dtype, 'isalignedstruct',
+                                                 is_aligned))
     elif dtype.kind == 'O':
         return object_
 
@@ -267,6 +279,13 @@ _dtypes = None
 def map_minitype_to_dtype(type):
     global _dtypes
 
+    if type.is_struct:
+        import numpy as np
+
+        fields = [(field_name, map_minitype_to_dtype(field_type))
+                      for field_name, field_type in type.fields]
+        return np.dtype(fields, align=not type.packed)
+
     if _dtypes is None:
         _dtypes = create_dtypes()
 
@@ -342,6 +361,9 @@ class Type(miniutils.ComparableObjectMixin):
     def comparison_type_list(self):
         return self.subtype_list
 
+    def _is_object(self, context):
+        return context.is_object(self)
+
     def __eq__(self, other):
         # Don't use isinstance here, compare on exact type to be consistent
         # with __hash__. Override where sensible
@@ -398,11 +420,26 @@ class Type(miniutils.ComparableObjectMixin):
         "Get a corresponding llvm type from this type"
         return context.to_llvm(self)
 
+    def to_ctypes(self):
+        import ctypes_conversion
+        return ctypes_conversion.convert_to_ctypes(self)
+
+    def get_dtype(self):
+        return map_minitype_to_dtype(self)
+
+    def is_string(self):
+        return self.is_c_string or self == char.pointer()
+
     def __getattr__(self, attr):
         if attr.startswith('is_'):
             return False
         return getattr(type(self), attr)
 
+    def __call__(self, *args):
+        """Return a FunctionType with return_type and args set
+        """
+        return FunctionType(self, args)
+
 class ArrayType(Type):
     """
     An array type. ArrayType may be sliced to obtain a subtype:
@@ -433,7 +470,8 @@ class ArrayType(Type):
 
     @property
     def comparison_type_list(self):
-        return [self.dtype, self.is_c_contig, self.is_f_contig, self.inner_contig]
+        return [self.dtype, self.is_c_contig, self.is_f_contig,
+                self.inner_contig, self.ndim]
 
     def pointer(self):
         raise Exception("You probably want a pointer type to the dtype")
@@ -520,7 +558,7 @@ class CArrayType(Type):
         self.size = size
 
     def __repr__(self):
-        return "%s[%d]" % (self.base_type, self.length)
+        return "%s[%d]" % (self.base_type, self.size)
 
     def to_llvm(self, context):
         return llvm.core.Type.array(self.base_type.to_llvm(context), self.size)
@@ -546,6 +584,8 @@ class NamedType(Type):
     def __eq__(self, other):
         return isinstance(other, NamedType) and self.name == other.name
 
+    __hash__ = Type.__hash__ # !@#$ py3k
+
     def __repr__(self):
         if self.qualifiers:
             return "%s %s" % (self.name, " ".join(self.qualifiers))
@@ -586,9 +626,16 @@ class IntType(NumericType):
     signed = True
     rank = 4
     itemsize = 4
+    typecode = None
 
     kind = INT_KIND
 
+    def __init__(self, typecode=None, **kwds):
+        super(IntType, self).__init__(**kwds)
+        self.typecode = typecode
+        if typecode is not None:
+            self.itemsize = struct_.calcsize(typecode)
+
     def to_llvm(self, context):
         if self.itemsize == 1:
             return lc.Type.int(8)
@@ -647,7 +694,7 @@ class Py_ssize_t_Type(IntType):
 
     def __init__(self, **kwds):
         super(Py_ssize_t_Type, self).__init__(**kwds)
-        self.itemsize = _plat_bits / 8
+        self.itemsize = getsize('c_ssize_t', _plat_bits // 8)
 
 class NPyIntp(IntType):
     is_numpy_intp = True
@@ -656,7 +703,6 @@ class NPyIntp(IntType):
 
     def __init__(self, **kwds):
         super(NPyIntp, self).__init__(**kwds)
-        import numpy as np
         ctypes_array = np.empty(0).ctypes.strides
         self.itemsize = ctypes.sizeof(ctypes_array._type_)
 
@@ -691,24 +737,89 @@ class ObjectType(Type):
     def __repr__(self):
         return "PyObject *"
 
+def pass_by_ref(type):
+    return type.is_struct or type.is_complex
+
+class Function(object):
+    """
+    Function types may be called with Python functions to create a Function
+    object. This may be used to minivect users for their own purposes. e.g.
+
+    @double(double, double)
+    def myfunc(...):
+       ...
+    """
+    def __init__(self, signature, py_func):
+        self.signature = signature
+        self.py_func = py_func
+
 class FunctionType(Type):
     subtypes = ['return_type', 'args']
     is_function = True
     is_vararg = False
 
+    struct_by_reference = False
+
+    def __init__(self, return_type, args, name=None, **kwds):
+        super(FunctionType, self).__init__(**kwds)
+        self.return_type = return_type
+        self.args = args
+        self.name = name
+
     def to_llvm(self, context):
         assert self.return_type is not None
+        self = self.actual_signature
         return lc.Type.function(self.return_type.to_llvm(context),
                                 [arg_type.to_llvm(context)
                                     for arg_type in self.args],
                                 self.is_vararg)
 
-    def __str__(self):
+    def __repr__(self):
         args = map(str, self.args)
         if self.is_vararg:
             args.append("...")
+        if self.name:
+            namestr = self.name + ' '
+        else:
+            namestr = ''
+        return "%s (*%s)(%s)" % (self.return_type, namestr, ", ".join(args))
 
-        return "%s (*)(%s)" % (self.return_type, ", ".join(args))
+    @property
+    def actual_signature(self):
+        """
+        Passing structs by value is not properly supported for different
+        calling conventions in LLVM, so we take an extra argument
+        pointing to a caller-allocated struct value.
+        """
+        if self.struct_by_reference:
+            args = []
+            for arg in self.args:
+                if pass_by_ref(arg):
+                    arg = arg.pointer()
+                args.append(arg)
+
+            return_type = self.return_type
+            if pass_by_ref(self.return_type):
+                return_type = void
+                args.append(self.return_type.pointer())
+
+            self = FunctionType(return_type, args)
+
+        return self
+
+    @property
+    def struct_return_type(self):
+        # Function returns a struct.
+        return self.return_type.pointer()
+
+    def __call__(self, *args):
+        if len(args) != 1 or isinstance(args[0], Type):
+            return super(FunctionType, self).__call__(*args)
+
+        assert self.return_type is not None
+        assert self.args is not None
+        func, = args
+        return Function(self, func)
 
 class VectorType(Type):
     subtypes = ['element_type']
@@ -721,6 +832,7 @@ class VectorType(Type):
                 element_type.itemsize in (4, 8)), element_type
         self.element_type = element_type
         self.vector_size = vector_size
+        self.itemsize = element_type.itemsize * vector_size
 
     def to_llvm(self, context):
         return lc.Type.vector(self.element_type.to_llvm(context),
@@ -730,7 +842,7 @@ class VectorType(Type):
     def comparison_type_list(self):
         return self.subtype_list + [self.vector_size]
 
-    def __str__(self):
+    def __repr__(self):
         itemsize = self.element_type.itemsize
         if self.element_type.is_float:
             if itemsize == 4:
@@ -743,6 +855,113 @@ class VectorType(Type):
             else:
                 raise NotImplementedError
 
+
+def _sort_types_key(field_type):
+    if field_type.is_complex:
+        return field_type.base_type.rank * 2
+    elif field_type.is_numeric or field_type.is_struct:
+        return field_type.rank
+    elif field_type.is_vector:
+        return _sort_key(field_type.element_type) * field_type.vector_size
+    elif field_type.is_carray:
+        return _sort_key(field_type.base_type) * field_type.size
+    elif field_type.is_pointer or field_type.is_object or field_type.is_array:
+        return 8
+    else:
+        return 1
+
+def _sort_key(keyvalue):
+    field_name, field_type = keyvalue
+    return _sort_types_key(field_type)
+
+def sort_types(types_dict):
+    # reverse sort on rank, forward sort on name
+    d = {}
+    for field in types_dict.iteritems():
+        key = _sort_key(field)
+        d.setdefault(key, []).append(field)
+
+    def key(keyvalue):
+        field_name, field_type = keyvalue
+        return field_name
+
+    fields = []
+    for rank in sorted(d, reverse=True):
+        fields.extend(sorted(d[rank], key=key))
+
+    return fields
+
+class struct(Type):
+    """
+    Create a struct type. Fields may be ordered or unordered. Unordered fields
+    will be ordered from big types to small types (for better alignment).
+
+    >>> struct([('a', int_), ('b', float_)], name='Foo') # ordered struct
+    struct Foo { int a, float b }
+    >>> struct(a=int_, b=float_, name='Foo') # unordered struct
+    struct Foo { float b, int a }
+    >>> struct(a=int32, b=int32, name='Foo') # unordered struct
+    struct Foo { int32 a, int32 b }
+
+    >>> struct(a=complex128, b=complex64, c=struct(f1=double, f2=double, f3=int32))
+    struct { struct { double f1, double f2, int32 f3 } c, complex128 a, complex64 b }
+    """
+
+    is_struct = True
+
+    def __init__(self, fields=(), name=None, readonly=False, packed=False, **kwargs):
+        super(struct, self).__init__()
+        if fields and kwargs:
+            raise minierror.InvalidTypeSpecification(
+                    "The struct must be either ordered or unordered")
+
+        if kwargs:
+            fields = sort_types(kwargs)
+
+        self.fields = list(fields)
+        self.rank = sum(_sort_key(field) for field in self.fields)
+        self.name = name
+        self.readonly = readonly
+        self.fielddict = dict(self.fields)
+        self.packed = packed
+
+    def __repr__(self):
+        if self.name:
+            name = self.name + ' '
+        else:
+            name = ''
+        return 'struct %s{ %s }' % (
+                name, ", ".join("%s %s" % (field_type, field_name)
+                                    for field_name, field_type in self.fields))
+
+    def to_llvm(self, context):
+        if self.packed:
+            lstruct = llvm.core.Type.packed_struct
+        else:
+            lstruct = llvm.core.Type.struct
+
+        return lstruct([field_type.to_llvm(context)
+                           for field_name, field_type in self.fields])
+
+    @property
+    def comparison_type_list(self):
+        return self.fields
+
+    def is_prefix(self, other_struct):
+        other_fields = other_struct.fields[:len(self.fields)]
+        return self.fields == other_fields
+
+    def add_field(self, name, type):
+        assert name not in self.fielddict
+        self.fielddict[name] = type
+        self.fields.append((name, type))
+
+def getsize(ctypes_name, default):
+    try:
+        return ctypes.sizeof(getattr(ctypes, ctypes_name))
+    except ImportError:
+        return default
+
 #
 ### Internal types
 #
@@ -752,21 +971,32 @@ void = VoidType()
 #
 ### Public types
 #
-Py_ssize_t = Py_ssize_t_Type()
-npy_intp = NPyIntp()
-size_t = IntType(name="size_t", rank=8.5, itemsize=8, signed=False)
-char = CharType(name="char")
-short = IntType(name="short", rank=2, itemsize=2)
-int_ = IntType(name="int", rank=4, itemsize=4)
-long_ = IntType(name="long", rank=5, itemsize=4)
-longlong = IntType(name="PY_LONG_LONG", rank=8, itemsize=8)
+try:
+    npy_intp = NPyIntp()
+except ImportError:
+    npy_intp = None
 
-uchar = CharType(name="unsigned char", signed=False)
-ushort = IntType(name="unsigned short", rank=2.5, itemsize=2, signed=False)
-uint = IntType(name="unsigned int", rank=4.5, itemsize=4, signed=False)
-ulong = IntType(name="unsigned long", rank=5.5, itemsize=4, signed=False)
-ulonglong = IntType(name="unsigned PY_LONG_LONG", rank=8.5, itemsize=8,
-                    signed=False)
+Py_ssize_t = Py_ssize_t_Type()
+Py_uintptr_t = IntType(name='Py_uintptr_t',
+                       itemsize=getsize('c_void_p', Py_ssize_t.itemsize),
+                       rank=8.5)
+
+
+size_t = IntType(name="size_t", rank=8.5,
+                 itemsize=getsize('c_size_t', _plat_bits / 8), signed=False)
+char = CharType(name="char", typecode='b')
+short = IntType(name="short", rank=2, typecode='h')
+int_ = IntType(name="int", rank=4, typecode='i')
+long_ = IntType(name="long", rank=5, typecode='l')
+longlong = IntType(name="PY_LONG_LONG", rank=8, typecode='q')
+
+uchar = CharType(name="unsigned char", signed=False, typecode='B')
+ushort = IntType(name="unsigned short", rank=2.5,
+                 typecode='H', signed=False)
+uint = IntType(name="unsigned int", rank=4.5, typecode='I', signed=False)
+ulong = IntType(name="unsigned long", rank=5.5, typecode='L', signed=False)
+ulonglong = IntType(name="unsigned PY_LONG_LONG", rank=8.5,
+                    typecode='Q', signed=False)
 
 bool_ = BoolType()
 object_ = ObjectType()
@@ -783,7 +1013,8 @@ uint64 = IntType(name="uint64", rank=8.5, signed=False, itemsize=8)
 
 float32 = float_ = FloatType(name="float", rank=10, itemsize=4)
 float64 = double = FloatType(name="double", rank=12, itemsize=8)
-float128 = longdouble = FloatType(name="long double", rank=14, itemsize=16)
+float128 = longdouble = FloatType(name="long double", rank=14,
+                                  itemsize=16)
 
 complex64 = ComplexType(name="complex64", base_type=float32,
                         rank=16, itemsize=8)
@@ -792,6 +1023,29 @@ complex128 = ComplexType(name="complex128", base_type=float64,
 complex256 = ComplexType(name="complex256", base_type=float128,
                          rank=20, itemsize=32)
 
+integral = []
+native_integral = []
+floating = []
+complextypes = []
+
+for typename in __all__:
+    minitype = globals()[typename]
+    if minitype.is_int:
+        integral.append(minitype)
+    elif minitype.is_float:
+        floating.append(minitype)
+    elif minitype.is_complex:
+        complextypes.append(minitype)
+
+numeric = integral + floating + complextypes
+native_integral.extend((Py_ssize_t, size_t))
+
+integral.sort(key=_sort_types_key)
+native_integral = [minitype for minitype in integral
+                                if minitype.typecode is not None]
+floating.sort(key=_sort_types_key)
+complextypes.sort(key=_sort_types_key)
+
 def get_utility():
     import numpy
 
