@@ -72,39 +72,25 @@ class TestForLoop(unittest.TestCase):
             self.assertEqual(result, 45)
             self.assertEqual(result, _for_loop_fn_0(test_data))
 
-    def test_compiled_for_loop_fn_0_float32(self):
-        test_data = numpy.array([1.2, 3.4, 5.6], dtype=numpy.float32)
-        compiled_for_loop_fn = numba_compile(
-            restype=numba.float32, argtypes=[numba.float32[:]])(for_loop_fn_0)
-        result = compiled_for_loop_fn(test_data)
-        control_result = test_data.sum()
-        self.assertLess(abs(result-control_result)/control_result, 1e-7)
-        self.assertEqual(result, for_loop_fn_0(test_data))
-
     def test_compiled_for_loop_fn_1(self):
-        compiled_for_loop_fn = numba_compile(argtypes=[i4, i4, i4],
-                                             restype=i4)(for_loop_fn_1)
-        result = compiled_for_loop_fn(1, 4, 1)
+        result = for_loop_fn_1(1, 4, 1)
         self.assertEqual(result, 6)
-        self.assertEqual(result, for_loop_fn_1(1, 4, 1))
+        self.assertEqual(result, _for_loop_fn_1(1, 4, 1))
 
     def test_compiled_for_loop_fn_2(self):
-        compiled_for_loop_fn = numba_compile(argtypes=[i4],
-                                             restype=i4)(for_loop_fn_2)
-        result = compiled_for_loop_fn(4)
+        result = for_loop_fn_2(4)
         self.assertEqual(result, 36)
-        self.assertEqual(result, for_loop_fn_2(4))
+        self.assertEqual(result, _for_loop_fn_2(4))
 
     def test_compiled_for_loop_fn_3(self):
-        compiled_for_loop_fn = numba_compile(argtypes=[i4],
-                                             restype=i4)(for_loop_fn_3)
-        result = compiled_for_loop_fn(3)
-        self.assertEqual(result, for_loop_fn_3(3))
+        result = for_loop_fn_3(3)
+        self.assertEqual(result, _for_loop_fn_3(3))
         self.assertEqual(result, 81)
 
 # ______________________________________________________________________
 
 if __name__ == "__main__":
+#    for_loop_fn_2(10)
     unittest.main()
 
 # ______________________________________________________________________
