@@ -1,7 +1,10 @@
+import sys
+import StringIO
 import unittest
 import functools
 
 from nose.tools import nottest
+import nose.plugins.skip
 from numba.decorators import jit as jit_, autojit
 
 import __builtin__
@@ -10,18 +13,14 @@ def checkSkipFlag(reason):
     def _checkSkipFlag(fn):
         @nottest
         def _checkSkipWrapper(self, *args, **kws):
-            if hasattr(__builtin__, '__noskip__'):
-                return fn(self, *args, **kws)
-            else:
-                self.skipTest(reason)
+            self.skipTest(reason)
         return _checkSkipWrapper
     return _checkSkipFlag
 
-class ByteCodeTestCase(unittest.TestCase):
-    jit = staticmethod(jit_)
-
-class ASTTestCase(ByteCodeTestCase):
+class ASTTestCase(unittest.TestCase):
     jit = staticmethod(lambda *args, **kw: jit_(*args, **dict(kw, backend='ast')))
+    backend = 'ast'
+    autojit = staticmethod(autojit(backend=backend))
 
 def main():
     import sys, logging
@@ -29,3 +28,14 @@ def main():
         logging.getLogger().setLevel(logging.DEBUG)
         sys.argv.remove('-d')
     unittest.main()
+
+class StdoutReplacer(object):
+    def __enter__(self, *args):
+        self.out = sys.stdout
+        sys.stdout = StringIO.StringIO()
+        return sys.stdout
+
+    def __exit__(self, *args):
+        sys.stdout = self.out
+
+from .bytecode.test_support import ByteCodeTestCase
\ No newline at end of file
