@@ -25,8 +25,9 @@ import logging
 logger = logging.getLogger(__name__)
 
 
-if __debug__:
-    import pprint
+debug = False
+
+import pprint
 
 _int32_zero = lc.Constant.int(_int32, 0)
 
@@ -95,7 +96,7 @@ def pythontype_to_strtype(typ):
 def map_to_strtype(type):
     "Map a minitype or str type to a str type"
     if isinstance(type, minitypes.Type):
-        if __debug__:
+        if debug:
             logger.debug('CONVERTING %r' % (type,))
         if type.is_float:
             if type.itemsize == 4:
@@ -131,7 +132,7 @@ def map_to_strtype(type):
             type = 'S'
         else:
             raise NotImplementedError(type)
-        if __debug__:
+        if debug:
             logger.debug(repr(type))
 
     return type
@@ -238,7 +239,7 @@ class Variable(object):
 
 # Add complex, unsigned, and bool
 def str_to_llvmtype(str):
-    if __debug__:
+    if debug:
         logger.debug("str_to_llvmtype(): str = %r" % (str,))
     n_pointer = 0
     if str.endswith('*'):
@@ -319,7 +320,7 @@ def convert_to_ctypes(typ):
     if typ.endswith('*'):
         n_pointer = typ.count('*')
         typ = typ[:-n_pointer]
-        if __debug__:
+        if debug:
             logger.debug("convert_to_ctypes(): n_pointer = %d, typ' = %r" %
                          (n_pointer, typ))
     dtype_str = np.dtype(typ).str
@@ -370,7 +371,7 @@ def typ_isa_number(typ):
 # Currently delegates casting to Variable.llvm(), but only in the
 # presence of a builder instance.
 def resolve_type(arg1, arg2, builder = None):
-    if __debug__:
+    if debug:
         logger.debug("arg1 = %r, arg2 = %r, builder = %r" % (arg1, arg2,
                                                              builder))
     typ = None
@@ -422,7 +423,7 @@ def resolve_type(arg1, arg2, builder = None):
         else:
             # Fall-through case: just use the left hand operand's type...
             typ = typ1
-    if __debug__:
+    if debug:
         logger.debug("resolve_type() ==> %r" % (typ,))
     return (typ,
             arg1.llvm(typ, builder = builder),
@@ -481,6 +482,9 @@ class _LLVMModuleUtils(object):
         'strlen' : lc.Type.function(_llvm_size_t, [_int8_star]),
         'strncpy' : lc.Type.function(_int8_star, [_int8_star, _int8_star,
                                                   _llvm_size_t]),
+        'strndup' : lc.Type.function(_int8_star, [_int8_star, _llvm_size_t]),
+        'malloc' : lc.Type.function(_void_star, [_llvm_size_t]),
+        'free' : lc.Type.function(lc.Type.void(), [_void_star]),
         }
 
     @classmethod
@@ -588,6 +592,22 @@ class _LLVMModuleUtils(object):
                                       "arrays.")
         return res, None
 
+    @classmethod
+    def build_int(cls, translator, args):
+        res = None
+        if len(args) >= 1 and len(args) <= 2:
+            if args[0].typ.startswith('S'):
+                res = translator.builder.call(
+                    _LLVMModuleUtils.get_lib_fn(translator.mod, 'atol'),
+                    [])
+                raise NotImplementedError('FIXME')
+            else:
+                raise NotImplementedError('Base conversions not currently '
+                                          'supported.')
+        else:
+            raise TypeError('int() takes at least 1 and at most 2 arguments')
+        return res, None
+
     @classmethod
     def get_py_incref(cls, module, py_obj_ptr_type = _void_star):
         return module.get_or_insert_function(
@@ -615,7 +635,7 @@ class _LLVMModuleUtils(object):
                 cls.get_py_incref(translator.mod),
                 [translator.builder.call(lfunc, largs)]),
             _numpy_array)
-        if __debug__:
+        if debug:
             logger.debug("lfunc = %r\nlargs =%r" % (
                     str(lfunc), [str(arg) for arg in largs]))
         return res, args[0].typ
@@ -712,9 +732,78 @@ class _LLVMModuleUtils(object):
             builder.ret(temp)
         return ret_val
 
+    @classmethod
+    def get_c_str_slice(cls, module):
+        ret_val = module.get_or_insert_function(
+            lc.Type.function(_int8_star, (_int8_star, _llvm_size_t,
+                                          _llvm_size_t)), '__c_str_slice')
+        if ret_val.is_declaration:
+            ret_val.linkage = lc.LINKAGE_INTERNAL
+            ret_val.args[0].name = 'in_str'
+            ret_val.args[1].name = 'lower'
+            ret_val.args[2].name = 'upper'
+            entry_block = ret_val.append_basic_block('entry')
+            lower_lt_0_block = ret_val.append_basic_block('lower_lt_0')
+            check_upper_block = ret_val.append_basic_block('check_upper')
+            upper_lt_0_block = ret_val.append_basic_block('upper_lt_0')
+            check_sublen_lb_block = ret_val.append_basic_block(
+                'check_sublen_lb')
+            sublen_lt_0_block = ret_val.append_basic_block('sublen_lt_0')
+            exit_block = ret_val.append_basic_block('exit')
+            zero = lc.Constant.int(_llvm_size_t, 0)
+            one = lc.Constant.int(_llvm_size_t, 1)
+            l_strlen = cls.get_lib_fn(module, 'strlen')
+            l_strndup = cls.get_lib_fn(module, 'strndup')
+            l_malloc = cls.get_lib_fn(module, 'malloc')
+            # entry:
+            builder = lc.Builder.new(entry_block)
+            in_str_len = builder.call(l_strlen, [ret_val.args[0]], 'in_str_len')
+            is_lower_lt_0 = builder.icmp(lc.ICMP_SLT, ret_val.args[1], zero,
+                                         'is_lower_lt_0')
+            builder.cbranch(is_lower_lt_0, lower_lt_0_block, check_upper_block)
+            # lower_lt_0:
+            builder = lc.Builder.new(lower_lt_0_block)
+            adj_lower = builder.add(ret_val.args[1], in_str_len, 'adj_lower')
+            builder.branch(check_upper_block)
+            # check_upper:
+            builder = lc.Builder.new(check_upper_block)
+            lower_phi = builder.phi(_llvm_size_t, 'lower_phi')
+            lower_phi.add_incoming(ret_val.args[1], entry_block)
+            lower_phi.add_incoming(adj_lower, lower_lt_0_block)
+            is_upper_lt_0 = builder.icmp(lc.ICMP_SLT, ret_val.args[2], zero,
+                                         'is_upper_lt_0')
+            builder.cbranch(is_upper_lt_0, upper_lt_0_block,
+                            check_sublen_lb_block)
+            # upper_lt_0:
+            builder = lc.Builder.new(upper_lt_0_block)
+            adj_upper = builder.add(ret_val.args[2], in_str_len, 'adj_upper')
+            builder.branch(check_sublen_lb_block)
+            # check_sublen_lb_block:
+            builder = lc.Builder.new(check_sublen_lb_block)
+            upper_phi = builder.phi(_llvm_size_t, 'upper_phi')
+            upper_phi.add_incoming(ret_val.args[2], check_upper_block)
+            upper_phi.add_incoming(adj_upper, upper_lt_0_block)
+            sublen = builder.sub(upper_phi, lower_phi, 'sublen')
+            is_sublen_lt_0 = builder.icmp(lc.ICMP_SLT, sublen, zero,
+                                          'is_sublen_lt_0')
+            builder.cbranch(is_sublen_lt_0, sublen_lt_0_block, exit_block)
+            # sublen_lt_0_block:
+            builder = lc.Builder.new(sublen_lt_0_block)
+            builder.branch(exit_block)
+            # exit_block
+            builder = lc.Builder.new(exit_block)
+            sublen_phi = builder.phi(_llvm_size_t, 'sublen_phi')
+            sublen_phi.add_incoming(sublen, check_sublen_lb_block)
+            sublen_phi.add_incoming(zero, sublen_lt_0_block)
+            in_str_start = builder.gep(ret_val.args[0], [lower_phi],
+                                      'in_str_start')
+            substr = builder.call(l_strndup, (in_str_start, sublen_phi))
+            builder.ret(substr)
+        return ret_val
+
     @classmethod
     def build_conj(cls, translator, args):
-        if __debug__:
+        if debug:
             logger.debug(repr(args))
         assert ((len(args) == 1) and (args[0]._llvm is not None) and
                 (args[0]._llvm.type in (_complex64, _complex128)))
@@ -732,6 +821,7 @@ class _LLVMModuleUtils(object):
 PY_CALL_TO_LLVM_CALL_MAP = {
     debugout : _LLVMModuleUtils.build_debugout,
     len : _LLVMModuleUtils.build_len,
+    int : _LLVMModuleUtils.build_int,
     np.zeros_like : _LLVMModuleUtils.build_zeros_like,
     np.complex64.conj : _LLVMModuleUtils.build_conj,
     np.complex128.conj : _LLVMModuleUtils.build_conj,
@@ -823,6 +913,9 @@ class LLVMControlFlowGraph (ControlFlowGraph):
 class Translate(object):
     def __init__(self, func, restype='d', argtypes=['d'], **kws):
         self.func = func
+        self.func_name = kws.get('name', None)
+        if not self.func_name:
+            self.func_name = self.func.func_name
         self.fco = func.func_code
         self.names = self.fco.co_names
         self.varnames = self.fco.co_varnames
@@ -854,7 +947,10 @@ class Translate(object):
         global _ObjectCache
         setattr(_ObjectCache, '_ObjectCache__instances', WeakValueDictionary())
 
-        self.mod = lc.Module.new('%s_mod_%x' % (func.__name__, id(self)))
+        if kws.has_key('module'):
+            self.mod = kws['module']
+        else:
+            self.mod = lc.Module.new('%s_mod_%x' % (func.__name__, id(self)))
         assert isinstance(self.mod, lc.Module), (
             "Expected %r from llvm-py, got instance of type %r, however." %
             (lc.Module, type(self.mod)))
@@ -886,7 +982,7 @@ class Translate(object):
         argnames = self.fco.co_varnames[:self.fco.co_argcount]
         self.arg_ltypes = [convert_to_llvmtype(x) for x in self.arg_types]
         ty_func = lc.Type.function(self.ret_ltype, self.arg_ltypes)
-        self.lfunc = self.mod.add_function(ty_func, self.func.func_name)
+        self.lfunc = self.mod.add_function(ty_func, self.func_name)
         assert isinstance(self.lfunc, lc.Function), (
             "Expected %r from llvm-py, got instance of type %r, however." %
             (lc.Function, type(self.lfunc)))
@@ -916,7 +1012,7 @@ class Translate(object):
         """
         self.cfg = LLVMControlFlowGraph.build_cfg(self.fco, self)
         self.cfg.compute_dataflow()
-        if __debug__:
+        if debug:
             logger.debug(self.cfg.pformat())
         is_dead_code = False
         # Filter out the OPNAME+x opcodes, assuming the handler knows
@@ -959,7 +1055,7 @@ class Translate(object):
             #fpm.run(self.lfunc)
             #fpm.finalize()
 
-        if __debug__:
+        if debug:
             logger.debug(str(self.mod))
 
     def _get_ee(self):
@@ -992,7 +1088,7 @@ class Translate(object):
                 lfunc_ptr_ty)
             lfunc_ptr_ptr.linkage = lc.LINKAGE_INTERNAL
         lfunc = target_translator.builder.load(lfunc_ptr_ptr)
-        if __debug__:
+        if debug:
             logger.debug(str(lfunc))
         largs = [arg.llvm(convert_to_strtype(param_typ),
                           builder = target_translator.builder)
@@ -1034,7 +1130,7 @@ class Translate(object):
                 raise NotImplementedError(
                     "Not currently supporting back patching casts for phi "
                     "connections.")
-        if __debug__:
+        if debug:
             logger.debug("value = %r, lval = %r, phi.type = %r" %
                          (value, str(lval), str(phi.type)))
         for pred_lblock in pred_lblocks:
@@ -1058,7 +1154,7 @@ class Translate(object):
 
         4. If the predecessor is unreachable, ignore it.
         '''
-        if __debug__:
+        if debug:
             logger.debug("crnt_block=%r, pred=%r, local=%r" %
                          (crnt_block, pred, local))
         if pred in self.blocks_locals and pred not in self.pending_blocks:
@@ -1068,13 +1164,13 @@ class Translate(object):
                 "already been visited.")
             lval = pred_locals[local].llvm(llvmtype_to_strtype(phi.type),
                                            builder = self.builder)
-            if __debug__:
+            if debug:
                 logger.debug("lval = %r, phi.type = %r" % (str(lval),
                                                            str(phi.type)))
             phi.add_incoming(lval, self.blocks[pred])
         elif 0 in self.cfg.blocks_reaching[pred]:
             reaching_defs = self.cfg.get_reaching_definitions(crnt_block)
-            if __debug__:
+            if debug:
                 logger.debug("reaching_defs = %s" %
                              (pprint.pformat(reaching_defs),))
             definition_block = reaching_defs[pred][local]
@@ -1092,11 +1188,11 @@ class Translate(object):
                     local]
                 self.add_pending_phi(definition_index, local, phi,
                                      self.blocks[pred])
-                if __debug__:
+                if debug:
                     logger.debug("self.pending_phis = %r" %
                                  (self.pending_phis,))
         else:
-            if __debug__:
+            if debug:
                 logger.debug("Block %d not reachable from entry." % pred)
 
     def build_phi_nodes(self, crnt_block):
@@ -1255,7 +1351,7 @@ class Translate(object):
         self.builder = lc.Builder.new(self.blocks[block_index])
         self._locals = self.blocks_locals[block_index]
         block = self.blocks[block_index]
-        if __debug__:
+        if debug:
             logger.debug(repr(block.instructions))
         instructions = [instruction for instruction in block.instructions
                         if not isinstance(instruction, lc.PHINode)]
@@ -1283,7 +1379,7 @@ class Translate(object):
         # Because the visitation order here is out of order, we have
         # to recheck for pending phis in the preceeding block, and
         # handle them.
-        if __debug__:
+        if debug:
             logger.debug(pprint.pformat((self._locals, self.pending_phis)))
         for local_index, local_val in enumerate(self._locals):
             if self.has_pending_phi(i - 3, local_index):
@@ -1363,7 +1459,7 @@ class Translate(object):
     def op_BINARY_ADD(self, i, op, arg):
         arg2 = self.stack.pop(-1)
         arg1 = self.stack.pop(-1)
-        if __debug__:
+        if debug:
             logger.debug("op_BINARY_ADD(): %r + %r" % (arg1, arg2))
         typ, arg1, arg2 = resolve_type(arg1, arg2, self.builder)
         if typ[0] == 'f':
@@ -1468,7 +1564,7 @@ class Translate(object):
     def op_BINARY_POWER(self, i, op, arg):
         arg2 = self.stack.pop(-1)
         arg1 = self.stack.pop(-1)
-        if __debug__:
+        if debug:
             logger.debug("%r ** %r" % (arg1, arg2))
         typ, larg1, larg2 = resolve_type(arg1, arg2, self.builder)
         if typ[0] == 'i':
@@ -1534,7 +1630,7 @@ class Translate(object):
             self.stack = self.stack[:-arg]
         func = self.stack.pop(-1)
         ret_typ = None
-        if __debug__:
+        if debug:
             logger.debug("func = %r" % (func,))
         if func.val in PY_CALL_TO_LLVM_CALL_MAP:
             res, ret_typ = PY_CALL_TO_LLVM_CALL_MAP[func.val](self, args)
@@ -1608,7 +1704,7 @@ class Translate(object):
 
     def op_LOAD_ATTR(self, i, op, arg):
         objarg = self.stack.pop(-1)
-        if __debug__:
+        if debug:
             logger.debug(repr((i, op, self.names[arg], objarg, objarg.typ)))
         if objarg.is_module():
             res = getattr(objarg.val, self.names[arg])
@@ -1617,7 +1713,7 @@ class Translate(object):
             # what typemap was destined to do...)
             objarg_llvm_val = objarg.llvm()
             res = None
-            if __debug__:
+            if debug:
                 logger.debug(str(objarg_llvm_val.type))
             if objarg_llvm_val.type == _numpy_array:
                 field_index = _numpy_array_field_ofs[self.names[arg]]
@@ -1754,19 +1850,19 @@ class Translate(object):
             lval = arr_var._llvm
             ltype = lval.type
             if ltype == _numpy_array:
-                if __debug__:
+                if debug:
                     logger.debug("op_BINARY_SUBSCR(): arr_var.typ = %s" %
                                  (arr_var.typ,))
                 result_val = self.builder.load(
                     self._build_pointer_into_arr_data(arr_var, index_var))
-                if __debug__:
+                if debug:
                     logger.debug(repr(result_val))
             elif ltype.kind == lc.TYPE_POINTER:
                 result_val = self.builder.load(
                     self.builder.gep(lval, [index_var.llvm(
                                 'i32', builder = self.builder)]))
             else:
-                if __debug__:
+                if debug:
                     logger.debug("op_BINARY_SUBSCR(): %r arr_var = %r (%r)\n%s"
                                  % ((i, op, arg), arr_var, str(arr_var._llvm),
                                     self.mod))
@@ -1776,7 +1872,7 @@ class Translate(object):
         elif isinstance(arr_var.val, tuple):
             raise NotImplementedError("FIXME")
         else:
-            if __debug__:
+            if debug:
                 logger.debug("op_BINARY_SUBSCR(): %r arr_var = %r (%r)\n%s" %
                              ((i, op, arg), arr_var, str(arr_var._llvm),
                               self.mod))
@@ -1793,13 +1889,13 @@ class Translate(object):
         index_var = self.stack.pop(-1)
         arr_var = self.stack.pop(-1)
         store_var = self.stack.pop(-1)
-        if __debug__:
+        if debug:
             logger.debug("%r\nop_STORE_SUBSCR(): %r[%r] = %r" % (
                     (i, op, arg), arr_var, index_var, store_var))
         if arr_var._llvm is not None:
             arr_lval = arr_var._llvm
             arr_ltype = arr_lval.type
-            if __debug__:
+            if debug:
                 logger.debug("op_STORE_SUBSCR(): arr_lval = '%s', "
                              "arr_ltype = '%s'" % (arr_lval, arr_ltype))
             if arr_ltype == _numpy_array:
@@ -1851,7 +1947,7 @@ class Translate(object):
         return self.op_BINARY_RSHIFT(i, op, arg)
 
     def op_BREAK_LOOP(self, i, op, arg):
-        if __debug__:
+        if debug:
             logger.debug("i = %r, op = %r, arg = %r, loop_stack = %r" %
                          (i, op, arg, self.loop_stack))
         loop_i, loop_arg = self.loop_stack[-1]
@@ -1874,20 +1970,40 @@ class Translate(object):
 
     def _apply_slice(self, indexable, lower = None, upper = None):
         ret_val = None
-        if __debug__:
+        if debug:
             logger.debug(repr((indexable, lower, upper)))
         if indexable.typ[0] == 'S':
-            llvm_indexable = indexable.llvm(builder = self.builder)
+            l_strlen = _LLVMModuleUtils.get_lib_fn(self.mod, 'strlen')
+            l_strncpy = _LLVMModuleUtils.get_lib_fn(self.mod, 'strncpy')
+            l_indexable = indexable.llvm(builder = self.builder)
             # calculate the alloca size.
+            size_t_typ = llvmtype_to_strtype(_llvm_size_t)
             if upper is None:
-                upper = self.builder.call(
-                    _LLVMModuleUtils.get_lib_fn(self.mod, 'strlen'), [
-                        llvm_indexable])
-            raise NotImplementedError("FIXME")
+                l_upper = self.builder.call(l_strlen, [l_indexable])
+            else:
+                l_upper = upper.llvm(size_t_typ, builder = self.builder)
+            if lower is None:
+                l_lower = lc.Constant.int(_llvm_size_t, 0)
+            else:
+                l_lower = lower.llvm(size_t_typ, builder = self.builder)
+            l_heap_result = self.builder.call(
+                _LLVMModuleUtils.get_c_str_slice(self.mod), [
+                    l_indexable, l_lower, l_upper])
+            substr_len = self.builder.call(l_strlen, [l_heap_result])
+            # XXX Might have issues on 64-bit systems.
+            l_result = self.builder.alloca_array(
+                _int8, self.builder.add(substr_len,
+                                        lc.Constant.int(_llvm_size_t, 1)))
+            self.builder.call(l_strncpy, [l_result, l_heap_result,
+                                          substr_len])
+            self.builder.call(
+                _LLVMModuleUtils.get_lib_fn(self.mod, 'free'),
+                [l_heap_result])
+            ret_val = Variable(l_result, None, 'S')
         else:
             raise NotImplementedError("Slices unsupported for type %r" %
                                       (indexable.typ,))
-        if __debug__:
+        if debug:
             logger.debug(repr(ret_val))
         return ret_val
 
